class SimpleClassWithBlankParentheses():
    pass
class ClassWithSpaceParentheses ( ):
    first_test_data = 90
    second_test_data = 100
    def test_func(self):
        return None
class ClassWithEmptyFunc(object):

    def func_with_blank_parentheses():
        return 5


def public_func_with_blank_parentheses():
    return None
def class_under_the_func_with_blank_parentheses():
    class InsideFunc():
        pass
class NormalClass (
):
    def func_for_testing(self, first, second):
        sum = first + second
        return sum


# output


class SimpleClassWithBlankParentheses:
    pass


class ClassWithSpaceParentheses:
    first_test_data = 90
    second_test_data = 100

    def test_func(self):
        return None


class ClassWithEmptyFunc(object):

    def func_with_blank_parentheses():
        return 5


def public_func_with_blank_parentheses():
    return None


def class_under_the_func_with_blank_parentheses():
    class InsideFunc:
        pass


class NormalClass:
    def func_for_testing(self, first, second):
        sum = first + second
        return sum
