#!/python

# regression test for #4762
"""
docstring
"""
from __future__ import annotations

import os

# output
#!/python

# regression test for #4762
"""
docstring
"""

from __future__ import annotations

import os
