# flags: --minimum-version=3.10
match something:
    case b(): print(1+1)
    case c(
        very_complex=True,
        perhaps_even_loooooooooooooooooooooooooooooooooooooong=-   1
    ): print(1)
    case c(
        very_complex=True,
        perhaps_even_loooooooooooooooooooooooooooooooooooooong=-1,
    ): print(2)
    case a: pass

match(
    arg # comment
)

match(
)

match(


)

case(
    arg # comment
)

case(
)

case(


)


re.match(
    something # fast
)
re.match(



)
match match(


):
    case case(
        arg, # comment
    ):
        pass

# output

match something:
    case b():
        print(1 + 1)
    case c(
        very_complex=True, perhaps_even_loooooooooooooooooooooooooooooooooooooong=-1
    ):
        print(1)
    case c(
        very_complex=True,
        perhaps_even_loooooooooooooooooooooooooooooooooooooong=-1,
    ):
        print(2)
    case a:
        pass

match(arg)  # comment

match()

match()

case(arg)  # comment

case()

case()


re.match(something)  # fast
re.match()
match match():
    case case(
        arg,  # comment
    ):
        pass
